#ifndef __Marzani_coeffs__
#define __Marzani_coeffs__

#include <fstream>
#include <vector>
#include <iostream>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_spline.h>
#include "finite-mt_small-x_coeffs.hh"
#include "hHiggs_new_var.hh"
#include <string>
//using namespace std;



const double mtop0 = 172.5;


SmallxCoeffs::SmallxCoeffs(std::string filename) {
  interp = true;
  if(filename == "") filename = "MBDFV.dat";
  std::ifstream infile(filename.c_str());
  std::vector<double> mH, c10, c20, c11, c30, c21;
  double d1, d2, d3, d4, d5, d6;
  do {
    infile >> d1 >> d2 >> d3 >> d4 >> d5 >> d6;
    mH.push_back(d1);
    c10.push_back(d2);
    c20.push_back(d3);
    c11.push_back(d4);
    c30.push_back(d5);
    c21.push_back(d6);
  } while(infile.good());
  mH.pop_back();
  c10.pop_back();
  c20.pop_back();
  c11.pop_back();
  c30.pop_back();
  c21.pop_back();
  //
  mN = mH.size();
  mmh  = new double[mN];
  mc10 = new double[mN];
  mc20 = new double[mN];
  mc11 = new double[mN];
  mc30 = new double[mN];
  mc21 = new double[mN];
  mc1  = new double[mN];
  mc2  = new double[mN];
  mc3  = new double[mN];
  for(int i=0; i<mN; i++) {
    mmh[i] = mH[i];
    mc10[i] = c10[i];
    mc20[i] = c20[i];
    mc11[i] = c11[i];
    mc30[i] = c30[i];
    mc21[i] = c21[i];
    // deprecated
    mc1[i] = 2*mc10[i];
    mc2[i] = 2*mc20[i] + mc11[i];
    mc3[i] = 2*(mc30[i] + mc21[i]);
  }
  infile.close();
}

SmallxCoeffs::SmallxCoeffs(double mH, double mtop, double mbot, double mcha) {
  interp = false;
  mN = 1;
  mmh  = new double[mN];
  mc10 = new double[mN];
  mc20 = new double[mN];
  mc11 = new double[mN];
  mc30 = new double[mN];
  mc21 = new double[mN];
  mc1  = new double[mN];
  mc2  = new double[mN];
  mc3  = new double[mN];
  HiggsSmallX hsx(mH,mtop,mbot,mcha);
  std::vector<std::vector<double> > hk = hsx.hCoeffs(3);
  for(int i=0; i<mN; i++) {
    mmh[i] = mH;
    mc10[i] = hk[1][0];
    mc20[i] = hk[2][0];
    mc11[i] = hk[2][1];
    mc30[i] = hk[3][0];
    mc21[i] = hk[3][1];
    // deprecated
    mc1[i] = 2*mc10[i];
    mc2[i] = 2*mc20[i] + mc11[i];
    mc3[i] = 2*(mc30[i] + mc21[i]);
  }
}
SmallxCoeffs::~SmallxCoeffs() {
  delete[] mmh;
  delete[] mc10;
  delete[] mc20;
  delete[] mc11;
  delete[] mc30;
  delete[] mc21;
  delete[] mc1;
  delete[] mc2;
  delete[] mc3;
}


double SmallxCoeffs::interpolation(double mH, double mtop, double *mc) {
  if(!interp) return mc[0];
  gsl_interp_accel *acc = gsl_interp_accel_alloc ();
  gsl_spline *spline = gsl_spline_alloc (gsl_interp_cspline, mN);
  gsl_spline_init (spline, mmh, mc, mN);
  double val = gsl_spline_eval (spline, mH*(mtop0/mtop), acc);
  gsl_spline_free (spline);
  gsl_interp_accel_free (acc);
  return val;
}
double SmallxCoeffs::c10(double mH, double mtop) {
  return interpolation(mH, mtop, mc10);
}
double SmallxCoeffs::c20(double mH, double mtop) {
  return interpolation(mH, mtop, mc20);
}
double SmallxCoeffs::c11(double mH, double mtop) {
  return interpolation(mH, mtop, mc11);
}
double SmallxCoeffs::c30(double mH, double mtop) {
  return interpolation(mH, mtop, mc30);
}
double SmallxCoeffs::c21(double mH, double mtop) {
  return interpolation(mH, mtop, mc21);
}

// deprecated
double SmallxCoeffs::c1marzani(double mH, double mtop) {
  return interpolation(mH, mtop, mc1);
}
double SmallxCoeffs::c2marzani(double mH, double mtop) {
  return interpolation(mH, mtop, mc2);
}
double SmallxCoeffs::c3marzani(double mH, double mtop) {
  return interpolation(mH, mtop, mc3);
}







#endif
